//+------------------------------------------------------------------+
//|               Coded by:  Gosta Hulden, MrMetatrader.com          |
//|                    web:  http//www.mrmetatrader.com              |
//|                  email:  gosta@mrmetatrader.com                  |
//+------------------------------------------------------------------+
#define LT    0
#define GT    1
#define GTE   2
#define LTE   3
#define EQ    4
#define NEQ   5


#property copyright "MoneyZilla"
#property link      "http://www.mrmetatrader.com"
#property strict
#include <Arrays\ArrayObj.mqh>
#include <stdlib.mqh>
//#property script_show_inputs
enum enumOrderType {
   Buy=OP_BUY, // Only buy trades
   Sell=OP_SELL, // Only sell trades
};
//----

extern     string CurrencyPair              = ""; // Currency pair (leave blank for current chart)
input     double MaxClosureSpread          = 0; // Max closure spread pips (0 = off)
input     double TargetPriceAbove          = 0; // Target price above (0 = No target above)
input     double TargetPriceBelow          = 0; // Target price below (0 = No target below)
input int            WritePlEveryXMin             = 240; // Write PL every X mins (0 = off)
input color          PlColour                     = clrYellow; // PL colour
input string SectionRt = "=== RT INPUT ==="; // === RT INPUTS ===
input int    MagicNumberRt = 2101;
input int    HardcodedSignalLabel = 1; // Hardcoded signal label no
input bool   CreateFreshTrades = true; // Create fresh trades/baskets
input color  ColorBuy = clrAqua;
input color  ColorSell = clrOrange;
input bool   ShowSignalCounter = false; // Show signal counter
input bool   SaveScreenshot = false; // Save screenshot (at signal time)
input bool   AlertAtSignal = false; // Alert when signal

/*input*/ int    MagicNumberCZ             = 999999; // Magic number
/*input*/ int    BrokerOffset              = 0; // Broker offset
/*input*/ string TradingStartTime          = "07:00"; // Trading start time
/*input*/ string TradingEndTime            = "16:00"; // Trading end time
/*input*/ double LotSize                   = 0.1; // Lot Size
/*input*/ bool   UseMoneyManagement        = true; // Use money management
/*input*/ double RiskPercentage            = 2; // Risk (percentage) 
/*input*/ double StopLossPips              = 60; // Stop loss (pips)
/*input*/ double TakeProfitPips            = 200; // Take profit (pips)
/*input*/ double MoveStopToBeAtPipsProfit  = 10; // Move stop to break even at x pips profit          
/*input*/ double TrailingStopTriggerPips   = 25; // Trailing stop profit trigger (pips)
/*input*/ double TrailingStopPips          = 25; // Trailing stop (pips)
          bool   ShowDashboard             = true; // Show dashboard
//input ENUM_ORDER_TYPE ot = OP_BUY; // Order type
/*input*/ enumOrderType ot = Buy; // Order type
 
string version = "1.20";
string EAName = "CloseZilla EA";
string globVarPrefix = "zi_";
string globVarPrefixRt = "rt_";
double pip, spread, marketLots[2], range, plOpen, avgPrice[2], lotsOpen[2];
int    orderCount[6], orderCountTot, orderType, mult, orderCountNet, orderCountNetPrev;
datetime lastCandle, lastInitTime;
string symb;
bool   initSuccess, firstTickAfterInit;

int    x = 3;
int    y = 15;
int    txtLine, txtLines, txtLinesPrev;


//+------------------------------------------------------------------+
//| INITIALIZATION                                                   |
//+------------------------------------------------------------------+
int OnInit() {
   initSuccess = true;
   Comment("Initializing " + EAName + " " + version + "...");
   Print("Initializing " + EAName + " " + version + "...");
   if (CurrencyPair == "") symb = Symbol();
   else {
      CurrencyPair = StringSubstr(CurrencyPair, 0, 6) + StringSubstr(Symbol(), 6);
      if (MarketInfo(CurrencyPair, MODE_ASK) == 0) {
         Alert(CurrencyPair + " is not a valid currency pair");
         Comment("Initialization failed: " + CurrencyPair + " is not a valid currency pair");
         initSuccess = false;
      }
      else symb = CurrencyPair;
   }
   if ((TargetPriceAbove <= Bid && TargetPriceAbove > 0) || (TargetPriceBelow >= Bid && TargetPriceBelow > 0)) {
      Alert("'Target price above' and/or 'Target price below' is invalid");
      Comment("Initialization failed: Target price above and/or Target price below is invalid");
      initSuccess = false;
   }
   pip = MarketInfo(symb,MODE_POINT);
   mult = 1;
   lastCandle = iTime(symb, Period(), 0);
   if (Digits == 3 || Digits == 5) {
      pip = pip * 10;
      mult = 10;
   }
   
   // Gold
   if (Bid > 500) {
      pip = 0.1;
      if (Digits == 3) mult = 100; else mult = 10;
   }
   
   if (IsTesting() == true) globVarPrefix = "bt"+globVarPrefix;
   if (IsTesting() == true) GlobalVariablesDeleteAll(globVarPrefix);
   
   lastInitTime = TimeCurrent();
   firstTickAfterInit = true;
   return(INIT_SUCCEEDED);
}

//+------------------------------------------------------------------+
//| DEINITIALIZATION                                                 |
//+------------------------------------------------------------------+
void OnDeinit(const int reason) {
   deleteText();
   ObjectDelete(0, "buttonCloseAll");
   ObjectDelete(0, "buttonCloseBuy");
   ObjectDelete(0, "buttonCloseSell");
   ObjectDelete(0, "buttonCloseAllPairs");
}

//+------------------------------------------------------------------+
//| MAIN LOOP                                                        |
//+------------------------------------------------------------------+
void OnTick() {
   if (initSuccess == false) return;
   Comment("");
   txtLine = 0;
   y = 15;    
   p(EAName + " " + version);
   spread = MarketInfo(symb, MODE_SPREAD) / mult;   
   checkOpenOrders();

   orderCountNet = orderCount[OP_BUY] - orderCount[OP_SELL];
   if (firstTickAfterInit == false) {
      if (orderCountNet != orderCountNetPrev) {
         if (orderCountNet > 0) {
            print("RT BUY SIGNAL. Order count net: " + IntegerToString(orderCountNet));
            rtSignal(OP_BUY, Bid);
         } 
         if (orderCountNet < 0) {
            print("RT SELL SIGNAL. Order count net: " + IntegerToString(orderCountNet));
            rtSignal(OP_SELL, Bid);
         } 
      }
   }
   orderCountNetPrev = orderCountNet;
   
   // CLOSURES
   // Target above
   if (lastInitTime > globVarGet("lastClosureTime") && orderCountTot > 0 && Bid >= TargetPriceAbove && TargetPriceAbove > 0) {
      Print("Closing all trades for " + symb + ". Price reached Target price above.");
      globVarSet("closeAll", 1);
   }
   // Target below
   if (lastInitTime > globVarGet("lastClosureTime") && orderCountTot > 0 && Bid <= TargetPriceBelow && TargetPriceBelow > 0) {
      Print("Closing all trades for " + symb + ". Price reached Target price below.");
      globVarSet("closeAll", 1);
   }

   // Reset
   if (orderCountTot == 0) {
      globVarSet("closeAll", 0);
      globVarSet("closeBuy", 0);
      globVarSet("closeSell", 0);  
   }
   if (orderCount[OP_BUY] + orderCount[OP_BUYSTOP] + orderCount[OP_BUYLIMIT] == 0) globVarSet("closeBuy", 0);
   if (orderCount[OP_SELL] + orderCount[OP_SELLSTOP] + orderCount[OP_SELLLIMIT] == 0) globVarSet("closeSell", 0);
   
   if (globVarGet("closeAll") > 0) {
      p("CLOSURE IS TRIGGERED!");
      openHedge();
      
      //Delete pending orders
      if (orderCount[OP_BUYSTOP]+orderCount[OP_BUYLIMIT]+orderCount[OP_SELLSTOP]+orderCount[OP_SELLLIMIT] > 0) {
         p("Closing pending orders...");
         closePendingOrders("Closed all pending orders");
      }
      if (NormalizeDouble(marketLots[OP_BUY], 2) == NormalizeDouble(marketLots[OP_SELL], 2) && orderCount[OP_BUY]+orderCount[OP_SELL] > 0) {
         if (spread <= MaxClosureSpread || MaxClosureSpread == 0) {
            p("Closing market orders...");
            closeMarketOrders("Closed all pending orders");
         }
         else p("POSTPONING CLOSURE! SPREAD IS TOO HIGH");
      } 
   }
   else if (globVarGet("closeBuy") > 0) {
      p("BUY CLOSURE IS TRIGGERED!");
      openHedge(OP_BUY);
      
      //Delete pending orders
      if (orderCount[OP_BUYSTOP]+orderCount[OP_BUYLIMIT] > 0) {
         p("Closing pending buy orders...");
         closePendingOrders("Closed all pending orders", OP_BUY);
      }
      if (/* NormalizeDouble(marketLots[OP_BUY], 2) == NormalizeDouble(marketLots[OP_SELL], 2) && */ orderCount[OP_BUY]+orderCount[OP_SELL] > 0) {
         if (spread <= MaxClosureSpread || MaxClosureSpread == 0) {
            p("Closing market buy orders...");
            closeMarketOrders("Closed all pending buyorders", OP_BUY);
         }
         else p("POSTPONING BUY CLOSURE! SPREAD IS TOO HIGH");
      } 
   } 
   else if (globVarGet("closeSell") > 0) {
      p("SELL CLOSURE IS TRIGGERED!");
      openHedge(OP_SELL);
      
      //Delete pending orders
      if (orderCount[OP_SELLSTOP]+orderCount[OP_SELLLIMIT] > 0) {
         p("Closing pending sell orders...");
         closePendingOrders("Closed all pending sell orders", OP_SELL);
      }
      if (/* NormalizeDouble(marketLots[OP_BUY], 2) == NormalizeDouble(marketLots[OP_SELL], 2) && */ orderCount[OP_BUY]+orderCount[OP_SELL] > 0) {
         if (spread <= MaxClosureSpread || MaxClosureSpread == 0) {
            p("Closing market buy orders...");
            closeMarketOrders("Closed all pending orders", OP_BUY);
         }
         else p("POSTPONING BUY CLOSURE! SPREAD IS TOO HIGH");
      } 
   } 

   p("Currency pair: " + symb);
   p("Market orders total (buy/sell): " + IntegerToString(orderCount[OP_BUY]+orderCount[OP_SELL]) + " (" + IntegerToString(orderCount[OP_BUY]) + "/" + IntegerToString(orderCount[OP_SELL]) + ")");
   p("Pending orders total (buy/sell): " + IntegerToString(orderCount[OP_BUYSTOP]+orderCount[OP_BUYLIMIT]+orderCount[OP_SELLSTOP]+orderCount[OP_SELLLIMIT]) + " (" + IntegerToString(orderCount[OP_BUYSTOP]+orderCount[OP_BUYLIMIT]) + "/" + IntegerToString(orderCount[OP_SELLSTOP]+orderCount[OP_SELLLIMIT]) + ")");
   p("Market Lots total (buy/sell): " + DoubleToString(marketLots[OP_BUY]+marketLots[OP_SELL], 2) + " (" + DoubleToString(marketLots[OP_BUY], 2) + "/" + DoubleToString(marketLots[OP_SELL], 2) + ")");
   p("B avg: "+ DoubleToString(avgPrice[OP_BUY], Digits) + " (" + DoubleToString(lotsOpen[OP_BUY], 2) + " lots)");
   p("S avg: "+ DoubleToString(avgPrice[OP_SELL], Digits) + " (" + DoubleToString(lotsOpen[OP_SELL], 2) + " lots)");
   p("P/L open: " + DoubleToString(plOpen, 2));
   p("Current time: " + getTimeString());
   p("Spread: " + DoubleToStr(spread, 1) + ", Max closure spread: " + DoubleToStr(MaxClosureSpread, 1));
   p("Target price above: " + DoubleToString(TargetPriceAbove, Digits) + ", Target price below: " + DoubleToString(TargetPriceBelow, Digits));


   
   printLines();
   
   createCloseAllButton();
    
   cleanUpText();
   printPl(PlColour);
   WindowRedraw();
   firstTickAfterInit = false;
   return;
}   
//////////////////////////////////////////////////////////////////////////
void checkOpenOrders () {
   for (int i=0; i<=5; i++) orderCount[i]=0;
   orderCountTot = 0;
   marketLots[OP_BUY] = 0;
   marketLots[OP_SELL] = 0;
   plOpen = 0;
   avgPrice[OP_BUY] = 0;
   avgPrice[OP_SELL] = 0;
   lotsOpen[OP_BUY] = 0;
   lotsOpen[OP_SELL] = 0;
   for(int i=OrdersTotal()-1; i>=0; i--) { 
      if (OrderSelect(i,SELECT_BY_POS)==true && OrderSymbol() == symb) {
         orderCount[OrderType()]++;
         orderCountTot++;
         if (OrderType() == OP_BUY || OrderType() == OP_SELL) {
            plOpen = plOpen + OrderProfit() + OrderCommission() + OrderSwap();
            lotsOpen[OrderType()] = lotsOpen[OrderType()] + OrderLots();
            avgPrice[OrderType()] = avgPrice[OrderType()] + (OrderOpenPrice()*OrderLots());
         }
         if (OrderType() == OP_BUY) {
            marketLots[OP_BUY] = marketLots[OP_BUY] + OrderLots();
         }
         else if (OrderType() == OP_SELL) {
            marketLots[OP_SELL] = marketLots[OP_SELL] + OrderLots();
         }
      }
   }
   if (lotsOpen[OP_BUY] > 0) avgPrice[OP_BUY] = avgPrice[OP_BUY] / lotsOpen[OP_BUY];
   if (lotsOpen[OP_SELL] > 0) avgPrice[OP_SELL] = avgPrice[OP_SELL] / lotsOpen[OP_SELL];
}
////////////////////////////////////////////////////////////////////
void p (string newString) {

   if (ShowDashboard == false) return;
   
   txtLine++;
   color colour;
   string font;
   if (txtLine == 1) {
      colour = PaleGreen;
      font = "Arial Black";
   } 
   else {
      colour = White;
      font = "Arial";
   }
   ObjectCreate("txt_bg"+IntegerToString(txtLine), OBJ_LABEL, 0, 0, 0);  
   ObjectSet("txt_bg"+IntegerToString(txtLine), OBJPROP_CORNER, 0); 
   ObjectSet("txt_bg"+IntegerToString(txtLine), OBJPROP_XDISTANCE, x); 
   ObjectSet("txt_bg"+IntegerToString(txtLine), OBJPROP_YDISTANCE, y);
   ObjectSetText("txt_bg"+IntegerToString(txtLine),"gggggggggggggggg",16, "Webdings", clrIndigo);

   ObjectCreate("txt_text"+IntegerToString(txtLine), OBJ_LABEL, 0, 0, 0);  
   ObjectSet("txt_text"+IntegerToString(txtLine), OBJPROP_CORNER, 0); 
   ObjectSet("txt_text"+IntegerToString(txtLine), OBJPROP_XDISTANCE, x+4); 
   ObjectSet("txt_text"+IntegerToString(txtLine), OBJPROP_YDISTANCE, y+4);
   ObjectSetText("txt_text"+IntegerToString(txtLine),newString,8, font, colour);
   
   y = y + 16;
}
//////////////////////////////////////////////////////////////////////
void deleteText () {
   string objectName;
   for (int i=ObjectsTotal()-1; i >= 0; i--) {
      objectName = ObjectName(i);
      if (StringSubstr(objectName, 0,4) == "txt_") {
         ObjectDelete(objectName); 
      }   
   }    
}
/////////////////////////////////////////////////////////////////////
void cleanUpText () {
   txtLines = txtLine;
   for (int i=txtLine+1; i<=txtLinesPrev; i++) {
      ObjectDelete("txt_bg"+IntegerToString(i)); 
      ObjectDelete("txt_text"+IntegerToString(i)); 
   }
   txtLinesPrev = txtLines;
}
////////////////////////////////////////////////////////////////////
void openOrder (int oType, double lotSize) {
   double openPrice=0;
   color  colour=clrBlue;
   int    ticket=0, err=0;
   
   for (int attempts = 1; attempts <= 20; attempts++) {
      RefreshRates();
      if (oType == OP_BUY) {
         openPrice = Ask;
         colour = Blue;
      }
      else if (oType == OP_SELL) {
         openPrice = Bid;
         colour = Red;
      }
      if (IsTradeAllowed() == true && IsTradeContextBusy() == false) 
         ticket = OrderSend(symb, oType, lotSize, openPrice, 100*mult, 0, 0, EAName, MagicNumberCZ, 0, colour);
      else {
         Sleep(2000);
         continue;
      }
      if (ticket > 0) {
         checkOpenOrders(); // SET SL/TP
         break;
      }
      err=GetLastError();
      if(err==4 || err==128 || err==136 || err==137 || err==138 || err == 141 || err == 145 || err == 146) //Busy errors
         Sleep(5000);
      else 
         break;
   }
   if (ticket < 0) {
      Print("======= ERROR OrderSend: "+ IntegerToString(err));         
      Print("======= oType: "+ IntegerToString(oType) + ", lotSize: "+ DoubleToString(lotSize, 2));         
      if (oType == OP_BUY || oType == OP_BUYLIMIT || oType == OP_BUYSTOP) Print("======= openPrice: "+ DoubleToString(openPrice, Digits) + ", Ask: "+ DoubleToString(Ask, Digits)); 
      else Print("======= openPrice: "+ DoubleToString(openPrice, Digits) + ", Bid: "+ DoubleToString(Bid, Digits));
   }
}
//////////////////////////////////////////////////////////////////////
void closePendingOrders (string closeMsg, int oType = 99) {
   bool somethingWasClosed = false;
   for (int i=OrdersTotal()-1; i>=0; i--) {
      if (OrderSelect(i,SELECT_BY_POS) && OrderSymbol() == symb) {       
         if (OrderType() >= 2) {
            if (oType == OP_BUY && (OrderType() == OP_SELLSTOP || OrderType() == OP_SELLLIMIT)) continue;
            if (oType == OP_SELL && (OrderType() == OP_BUYSTOP || OrderType() == OP_BUYLIMIT)) continue;
            somethingWasClosed = OrderDelete(OrderTicket(), clrOrange);
         }
      }
   }
   if (somethingWasClosed == true) Print("================ CLOSE: " + closeMsg);
}

//////////////////////////////////////////////////////////////////////
bool checkSessionOpen () {
   if (TradingStartTime <= TradingEndTime) {
      if (getTimeString() >= TradingStartTime && getTimeString() <= TradingEndTime)
         return (true);
      else
         return(false);
   }
   else {
      if (getTimeString() >= TradingStartTime || getTimeString() <= TradingEndTime)
         return (true);
      else
         return(false);
   }      
}

//////////////////////////////////////////////////////////////////////
double fixLotSize (double pLotSize) {
      double lotStep = MarketInfo(symb, MODE_LOTSTEP);
      double lotSize = pLotSize;
      // Round the decimals down (not to over trade) to make it an even micro/mini/standard lot
      lotSize = lotSize / lotStep;
      lotSize = MathRound(lotSize);
      lotSize = lotSize * lotStep;
      if (lotSize > MarketInfo(symb, MODE_MAXLOT))
         lotSize = MarketInfo(symb, MODE_MAXLOT);
      return(lotSize);
}
/////////////////////////////////////////////////////////////////////////
double iif (bool condition, double ifTrue, double ifFalse ) {
    if (condition == true) return( ifTrue );
    else return( ifFalse );
}
string iifStr (bool condition, string ifTrue, string ifFalse ) {
    if( condition == true) return( ifTrue );
    else return( ifFalse );
}
int iifInt (bool condition, int ifTrue, int ifFalse ) {
    if (condition == true) return( ifTrue );
    else return( ifFalse );
}
datetime iifDt (bool condition, datetime ifTrue, datetime ifFalse ) {
    if (condition == true) return( ifTrue );
    else return( ifFalse );
}
////////////////////////////////////////////////////////////////////
datetime timeCurrent () {
   return(TimeCurrent()-3600*BrokerOffset);
}
int hour () {
   return (TimeHour(timeCurrent()));
}
int day () {
   return(TimeDay(timeCurrent()));
}
string getTimeString () {
   datetime dt = timeCurrent();
   return (TimeToStr(dt, TIME_MINUTES));
}
/////////////////////////////////////////////////////////////////////
void print (string s) {
   if (true) Print(TimeToString(TimeCurrent(), TIME_SECONDS) + ": " + s);
}
////////////////////////////////////////////////////////////////////
void globVarSet (string name, double value) {
   GlobalVariableSet(globVarPrefix + symb + "_" + name, value);
}
double globVarGet (string name) {
   return (GlobalVariableGet(globVarPrefix + symb + "_" + name));
}
bool globVarCheck (string name) {
   bool returnVal = GlobalVariableCheck(globVarPrefix + symb + "_" + name);
   return (returnVal);
}
void globVarSetRt (string name, double value) {
   GlobalVariableSet(globVarPrefixRt + symb +"_"+IntegerToString(MagicNumberRt) + "_" + name, value);
}
double globVarGetRt (string name) {
   return (GlobalVariableGet(globVarPrefixRt + symb +"_"+IntegerToString(MagicNumberRt) + "_" + name));
}
///////////////////////////////////////////////////////////////////
void createCloseAllButton () {
   string objName;
   long   chartId = 0;
   ResetLastError();
   objName = "buttonCloseAll";
   if (ObjectFind(0, "buttonCloseAll") < 0) {
      if( ObjectCreate(chartId, objName, OBJ_BUTTON, 0, 0, 0) == false) {
         Print(__FUNCTION__, ": failed to create the button! Error code = ",GetLastError());
         return;
      }
   }
   ObjectSetInteger(chartId, objName, OBJPROP_XDISTANCE, 3);
   ObjectSetInteger(chartId, objName, OBJPROP_YDISTANCE, y + 16);
   ObjectSetInteger(chartId, objName, OBJPROP_XSIZE, 134);
   ObjectSetInteger(chartId, objName, OBJPROP_YSIZE, 20);
   ObjectSetInteger(chartId, objName, OBJPROP_CORNER, CORNER_LEFT_UPPER);
   ObjectSetString(chartId, objName, OBJPROP_TEXT, "CLOSE ALL " + symb);
   ObjectSetString(chartId, objName, OBJPROP_FONT, "Arial");
   ObjectSetInteger(chartId, objName, OBJPROP_FONTSIZE, 8);
   ObjectSetInteger(chartId, objName, OBJPROP_COLOR, clrWhite);
   ObjectSetInteger(chartId, objName, OBJPROP_BGCOLOR, clrRed);
   ObjectSetInteger(chartId, objName, OBJPROP_BORDER_COLOR, clrGray);
   ObjectSetInteger(chartId, objName, OBJPROP_BACK, false);
   ObjectSetInteger(chartId, objName, OBJPROP_STATE, false);
   ObjectSetInteger(chartId, objName, OBJPROP_SELECTABLE, false);
   ObjectSetInteger(chartId, objName, OBJPROP_SELECTED, false);
   ObjectSetInteger(chartId, objName, OBJPROP_HIDDEN, false);
   ObjectSetInteger(chartId, objName, OBJPROP_ZORDER, 0);

   objName = "buttonCloseBuy";
   if (ObjectFind(0, "buttonCloseBuy") < 0) {
      if( ObjectCreate(chartId, objName, OBJ_BUTTON, 0, 0, 0) == false) {
         Print(__FUNCTION__, ": failed to create the button! Error code = ",GetLastError());
         return;
      }
   }
   ObjectSetInteger(chartId, objName, OBJPROP_XDISTANCE, 140);
   ObjectSetInteger(chartId, objName, OBJPROP_YDISTANCE, y + 16);
   ObjectSetInteger(chartId, objName, OBJPROP_XSIZE, 90);
   ObjectSetInteger(chartId, objName, OBJPROP_YSIZE, 20);
   ObjectSetInteger(chartId, objName, OBJPROP_CORNER, CORNER_LEFT_UPPER);
   ObjectSetString(chartId, objName, OBJPROP_TEXT, "CLOSE BUY");
   ObjectSetString(chartId, objName, OBJPROP_FONT, "Arial");
   ObjectSetInteger(chartId, objName, OBJPROP_FONTSIZE, 8);
   ObjectSetInteger(chartId, objName, OBJPROP_COLOR, clrWhite);
   ObjectSetInteger(chartId, objName, OBJPROP_BGCOLOR, clrRed);
   ObjectSetInteger(chartId, objName, OBJPROP_BORDER_COLOR, clrGray);
   ObjectSetInteger(chartId, objName, OBJPROP_BACK, false);
   ObjectSetInteger(chartId, objName, OBJPROP_STATE, false);
   ObjectSetInteger(chartId, objName, OBJPROP_SELECTABLE, false);
   ObjectSetInteger(chartId, objName, OBJPROP_SELECTED, false);
   ObjectSetInteger(chartId, objName, OBJPROP_HIDDEN, false);
   ObjectSetInteger(chartId, objName, OBJPROP_ZORDER, 0);

   objName = "buttonCloseSell";
   if (ObjectFind(0, "buttonCloseSell") < 0) {
      if( ObjectCreate(chartId, objName, OBJ_BUTTON, 0, 0, 0) == false) {
         Print(__FUNCTION__, ": failed to create the button! Error code = ",GetLastError());
         return;
      }
   }
   ObjectSetInteger(chartId, objName, OBJPROP_XDISTANCE, 233);
   ObjectSetInteger(chartId, objName, OBJPROP_YDISTANCE, y + 16);
   ObjectSetInteger(chartId, objName, OBJPROP_XSIZE, 90);
   ObjectSetInteger(chartId, objName, OBJPROP_YSIZE, 20);
   ObjectSetInteger(chartId, objName, OBJPROP_CORNER, CORNER_LEFT_UPPER);
   ObjectSetString(chartId, objName, OBJPROP_TEXT, "CLOSE SELL");
   ObjectSetString(chartId, objName, OBJPROP_FONT, "Arial");
   ObjectSetInteger(chartId, objName, OBJPROP_FONTSIZE, 8);
   ObjectSetInteger(chartId, objName, OBJPROP_COLOR, clrWhite);
   ObjectSetInteger(chartId, objName, OBJPROP_BGCOLOR, clrRed);
   ObjectSetInteger(chartId, objName, OBJPROP_BORDER_COLOR, clrGray);
   ObjectSetInteger(chartId, objName, OBJPROP_BACK, false);
   ObjectSetInteger(chartId, objName, OBJPROP_STATE, false);
   ObjectSetInteger(chartId, objName, OBJPROP_SELECTABLE, false);
   ObjectSetInteger(chartId, objName, OBJPROP_SELECTED, false);
   ObjectSetInteger(chartId, objName, OBJPROP_HIDDEN, false);
   ObjectSetInteger(chartId, objName, OBJPROP_ZORDER, 0);

   objName = "buttonCloseAllPairs";
   if (ObjectFind(0, "buttonCloseAllPairs") < 0) {
      if( ObjectCreate(chartId, objName, OBJ_BUTTON, 0, 0, 0) == false) {
         Print(__FUNCTION__, ": failed to create the button! Error code = ",GetLastError());
         return;
      }
   }
   ObjectSetInteger(chartId, objName, OBJPROP_XDISTANCE, 326);
   ObjectSetInteger(chartId, objName, OBJPROP_YDISTANCE, y + 16);
   ObjectSetInteger(chartId, objName, OBJPROP_XSIZE, 110);
   ObjectSetInteger(chartId, objName, OBJPROP_YSIZE, 20);
   ObjectSetInteger(chartId, objName, OBJPROP_CORNER, CORNER_LEFT_UPPER);
   ObjectSetString(chartId, objName, OBJPROP_TEXT, "CLOSE ALL PAIRS");
   ObjectSetString(chartId, objName, OBJPROP_FONT, "Arial");
   ObjectSetInteger(chartId, objName, OBJPROP_FONTSIZE, 8);
   ObjectSetInteger(chartId, objName, OBJPROP_COLOR, clrWhite);
   ObjectSetInteger(chartId, objName, OBJPROP_BGCOLOR, clrRed);
   ObjectSetInteger(chartId, objName, OBJPROP_BORDER_COLOR, clrGray);
   ObjectSetInteger(chartId, objName, OBJPROP_BACK, false);
   ObjectSetInteger(chartId, objName, OBJPROP_STATE, false);
   ObjectSetInteger(chartId, objName, OBJPROP_SELECTABLE, false);
   ObjectSetInteger(chartId, objName, OBJPROP_SELECTED, false);
   ObjectSetInteger(chartId, objName, OBJPROP_HIDDEN, false);
   ObjectSetInteger(chartId, objName, OBJPROP_ZORDER, 0);

}
/////////////////////////////////////////////////////////////////////////////
void openHedge (int oType=99) {
   int    type = -1;
   double lots = 0;
   if (marketLots[OP_BUY] > marketLots[OP_SELL] || oType == OP_BUY) type = OP_SELL;
   else if (marketLots[OP_SELL] > marketLots[OP_BUY] || oType == OP_SELL) type = OP_BUY;
   if (oType == OP_BUY) lots = marketLots[OP_BUY];
   else if (oType == OP_SELL) lots = marketLots[OP_SELL];
   else lots = fixLotSize(MathAbs(marketLots[OP_BUY]-marketLots[OP_SELL]));
   if (lots > 0) {
      openOrder(type, lots);
      checkOpenOrders();
   }
}
/////////////////////////////////////////////////////////////////////////////
void closeMarketOrders (string closeMsg, int oType = 99) {
   bool somethingWasClosed = false;
   int ticketBuy = 0;
   int ticketSell = 0;
   while (orderCount[OP_BUY] + orderCount[OP_SELL] > 0 && (spread <= MaxClosureSpread || MaxClosureSpread == 0)) {
      if (orderCount[OP_BUY] == 0 && oType == OP_BUY) break;
      if (orderCount[OP_SELL] == 0 && oType == OP_SELL) break;
      for (int i=OrdersTotal()-1; i>=0; i--) {
         spread = MarketInfo(symb, MODE_SPREAD) / mult;
         if (spread > MaxClosureSpread && MaxClosureSpread > 0) continue;
         if (OrderSelect(i,SELECT_BY_POS) && OrderSymbol() == symb) {       
            if (oType == OP_BUY && OrderType() == OP_SELL && OrderMagicNumber() != MagicNumberCZ) continue;
            if (oType == OP_SELL && OrderType() == OP_BUY && OrderMagicNumber() != MagicNumberCZ) continue;
            if (OrderType() == OP_BUY && ticketBuy == 0) ticketBuy = OrderTicket();
            else if (OrderType() == OP_SELL && ticketSell == 0) ticketSell = OrderTicket();
            if (ticketBuy > 0 && ticketSell > 0) {
               somethingWasClosed = OrderCloseBy(ticketBuy, ticketSell, clrOrange);
               if (somethingWasClosed == false) {
                  Print("ERROR closing market trades, Error: " + IntegerToString(GetLastError()));
               }
               ticketBuy = 0;
               ticketSell = 0;
               RefreshRates();               
            }
         }
      }
      checkOpenOrders();
   }
   if (somethingWasClosed == true) Print("================ CLOSE: " + closeMsg);
}
//+------------------------------------------------------------------+
//| ChartEvent function                                              |
//+------------------------------------------------------------------+
void OnChartEvent(const int id,
                  const long &lparam,
                  const double &dparam,
                  const string &sparam) {
   if(id == CHARTEVENT_OBJECT_CLICK){
      if(sparam=="buttonCloseAll") {
         ObjectSetInteger(0,"buttonCloseAll", OBJPROP_STATE, true);
         int ret=MessageBox("Click on OK if you want to delete all trade for this currency pair", "CONFIRM CLOSE ALL", MB_OKCANCEL); // Message box
         if(ret == IDOK) {
            globVarSet("closeAll", 1);
         }
         ObjectSetInteger(0,"buttonCloseAll", OBJPROP_STATE, false);
      }      
      if(sparam=="buttonCloseBuy") {
         ObjectSetInteger(0,"buttonCloseBuy", OBJPROP_STATE, true);
         int ret=MessageBox("Click on OK if you want to close all BUY trades for this currency pair", "CONFIRM CLOSE ALL BUY", MB_OKCANCEL); // Message box
         if(ret == IDOK) {
            globVarSet("closeBuy", 1);
         }
         ObjectSetInteger(0,"buttonCloseBuy", OBJPROP_STATE, false);
      }      
      if(sparam=="buttonCloseSell") {
         ObjectSetInteger(0,"buttonCloseSell", OBJPROP_STATE, true);
         int ret=MessageBox("Click on OK if you want to close all SELL trades for this currency pair", "CONFIRM CLOSE ALL SELL", MB_OKCANCEL); // Message box
         if(ret == IDOK) {
            globVarSet("closeSell", 1);
         }
         ObjectSetInteger(0,"buttonCloseSell", OBJPROP_STATE, false);
      }      
      if(sparam=="buttonCloseAllPairs") {
         ObjectSetInteger(0,"buttonCloseAllPairs", OBJPROP_STATE, true);
         int ret=MessageBox("Click on OK if you want to close ALL PAIRS", "CONFIRM CLOSE ALL PAIRS", MB_OKCANCEL); // Message box
         if(ret == IDOK) {
            closeAllPairs();
         }
         ObjectSetInteger(0,"buttonCloseAllPairs", OBJPROP_STATE, false);
      }      
   }
}
/////////////////////////////////////////////////////////////////////////////
void rtSignal (int oType, double price) {
   string alertMsg, objNameTxt;
   if (oType == OP_BUY) {
      globVarSetRt("rtLatestBuy", price);
      globVarSetRt("rtLatestBuyTime", TimeCurrent());
      if (ShowSignalCounter) {
         globVarSetRt("seq", globVarGetRt("seq") + 1);
         objNameTxt = "BuyLabel_"+IntegerToString(TimeCurrent())+"_"+DoubleToString(price, Digits)+"_"+IntegerToString(MagicNumberRt)+"_";
         ObjectCreate(objNameTxt, OBJ_TEXT, 0, TimeCurrent(), price);
         ObjectSetText(objNameTxt, DoubleToString(globVarGetRt("seq"), 0), 8, "Arial Black", ColorBuy);
      }
      globVarSetRt("rtLatestBuySeq", HardcodedSignalLabel);
      globVarSetRt("rtCreateFreshTradeBuy", iif(CreateFreshTrades, 1, 0));
      alertMsg = "CloseZilla Buy signal";
   }
   else {
      globVarSetRt("rtLatestSell", price);
      globVarSetRt("rtLatestSellTime", TimeCurrent());
      if (ShowSignalCounter) {
         globVarSetRt("seq", globVarGetRt("seq") + 1);
         objNameTxt = "SellLabel_"+IntegerToString(TimeCurrent())+"_"+DoubleToString(price, Digits)+"_"+IntegerToString(MagicNumberRt)+"_";
         ObjectCreate(objNameTxt, OBJ_TEXT, 0, TimeCurrent(), price);
         ObjectSetText(objNameTxt, DoubleToString(globVarGetRt("seq"), 0), 8, "Arial Black", ColorSell);
      }
      globVarSetRt("rtLatestSellSeq", HardcodedSignalLabel);
      globVarSetRt("rtCreateFreshTradeSell", iif(CreateFreshTrades, 1, 0));
      alertMsg = "CloseZilla Sell signal";
   }
   
   // Alert
   if (AlertAtSignal == true) {
      Alert(TimeToString(TimeCurrent(), TIME_SECONDS) + " " + Symbol() + ": " + alertMsg + " " + DoubleToString(price, Digits));
   }

   // Screenshot   
   if (SaveScreenshot == true) {
      string gifName = "CloseZilla_Screenshots\\"+Symbol()+"\\"+TimeToString(TimeCurrent(), TIME_DATE|TIME_SECONDS)+".gif";
      StringReplace(gifName, ":", ".");
      if (WindowScreenShot(gifName, 1024, 768) == false) {
         Print("Error creating file: Error code: " + IntegerToString(GetLastError()));
      }  
   }
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
class OpenTradeSymbol : public CObject
{
protected:
   string m_symbol;
public:
   OpenTradeSymbol(string symbol):m_symbol(symbol){}
   double Size()const 
   {
      double net_size=0.0;
      for(int i=OrdersTotal()-1;i>=0;i--)
      {
         if(OrderSelect(i,SELECT_BY_POS)&&OrderSymbol()==m_symbol)
         {
            if(OrderType()==OP_BUY)
               net_size+=OrderLots();
            else
            if(OrderType()==OP_SELL)
               net_size-=OrderLots();
         }  
      } 
      return net_size;  
   }
   string Symbol()   { return m_symbol;}
   int Compare(const CObject *node,const int mode=0)const override
   {
      OpenTradeSymbol* other = (OpenTradeSymbol*)node;
      if(fabs(this.Size())>fabs(other.Size()))return -1;
      if(fabs(this.Size())<fabs(other.Size()))return 1;
      return 0;
   }
};
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
class PendingOrder : public CObject
{
protected:
   int m_ticket;
public:
   PendingOrder(int ticket):m_ticket(ticket){}
   int Ticket(){return m_ticket;}
   int Points()const
   {
      if(!OrderSelect(m_ticket,SELECT_BY_TICKET))
         return 0;
      double _point = SymbolInfoDouble(OrderSymbol(),SYMBOL_POINT);
      double bid = SymbolInfoDouble(OrderSymbol(),SYMBOL_BID);
      return int(fabs(OrderOpenPrice()-bid));
   }
   int Compare(const CObject *node,const int mode=0)const override
   {
      PendingOrder* other = (PendingOrder*)node;
      if(fabs(this.Points())>fabs(other.Points()))return 1;
      if(fabs(this.Points())<fabs(other.Points()))return -1;
      return 0;
   }
};

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
class TradeManager : public CArrayObj
{
public:
   OpenTradeSymbol* operator[](int i){return At(i);}
   void HedgeAll()
   {
      int num_open = 0;
      ulong ms = GetTickCount();
      for(int i=OrdersTotal()-1;i>=0;i--)
      {
         if(OrderSelect(i,SELECT_BY_POS)&&OrderType()<2)
         {
            num_open++;
            bool found = false;
            for(int j=0;j<Total();j++)
            {   
               if(this[j].Symbol()==OrderSymbol())
               {
                  found=true;
                  break;     
               }
            }
            if(!found)
            {
               Add(new OpenTradeSymbol(OrderSymbol()));
            }
         }
      }
      Sort();
      int attempts = 0;
      bool is_hedged = false;
      int num_symbols = Total();
      while(!is_hedged&&!IsStopped()&&attempts<3)
      {
         attempts++;
         for(int i=0;i<Total();i++)
         {
            double size = NormalizeDouble(this[i].Size(),2);
            string symbol = this[i].Symbol();
            if(size > 0.0)
            {
               double price = SymbolInfoDouble(symbol,SYMBOL_BID);
               if(OrderSend(symbol,OP_SELL,size,price,0,0,0)<0)
                  Print("HedgeOrderError: ",ErrorDescription(GetLastError()));
            }
            else
            if(size < 0.0)
            {
               double price = SymbolInfoDouble(symbol,SYMBOL_ASK);
               if(OrderSend(symbol,OP_BUY,fabs(size),price,0,0,0)<0)
                  Print("HedgeOrderError: ",ErrorDescription(GetLastError()));
            }
         }
         is_hedged=IsHedged();
      } 
      ms = GetTickCount()-ms;
      if(is_hedged)
      {
         Alert(string(num_open)+" trades on "+string(num_symbols)+" symbols closed in "+string(ms)+" ms. Order reconciliation will happen after any pending orders are closed.");
      }
   }
   void CloseByHedge()
   {
      for(int s=0;s<Total();s++)
      {
         string symbol = this[s].Symbol();
         for(int i=OrdersTotal()-1;i>0&&!IsStopped();i--)
         {
            if(OrderSelect(i,SELECT_BY_POS)
               && OrderSymbol()==symbol 
               && OrderType()<2 
               //&& (!m_locked_magic || OrderMagicNumber() == MAGIC)
               )
            {
               int type = OrderType();
               int ticket = OrderTicket();
               for(int j=i-1;OrderSelect(j,SELECT_BY_POS);j--)
               {
                  if(OrderSymbol()==symbol 
                     && OrderType()<2 
                     && OrderType()!=type 
                     //&& (!m_locked_magic || OrderMagicNumber() == MAGIC)
                     )
                  {
                     if(!OrderCloseBy(OrderTicket(),ticket))
                        Print("OrderCLoseByError: ",ErrorDescription(GetLastError()));
                     i=OrdersTotal();
                     break; 
                  }
               }
            }
         }   
      }
   }
   void ClosePending()
   {
      int attempts = 0;
      while(!IsStopped()&&!AllPendingClosed()&&attempts<3)
      {
         CArrayObj list;
         for(int i=OrdersTotal();i>=0;i--)
         {
            if(OrderSelect(i,SELECT_BY_POS)&&OrderType()>=2)
               list.Add(new PendingOrder(OrderTicket()));
         }
         list.Sort();
         int i=0;
         for(PendingOrder *ord=list.At(i);ord!=NULL;ord=list.At(++i))
         {
            if(!OrderDelete(ord.Ticket()))
               Print("OrderDeleteError: ",ErrorDescription(GetLastError()));   
         }
         attempts++;
      }
   }
protected:
   bool  IsHedged()
   {
      for(int i=0;i<Total();i++)
         if(NormalizeDouble(this[i].Size(),2)!=0.0)
            return false;
      return true;
   }
   bool  AllPendingClosed()
   {
      for(int i=OrdersTotal();i>=0;i--)
         if(OrderSelect(i,SELECT_BY_POS)&&OrderType()>=2)
            return false;
      return true;
   }
};
//+------------------------------------------------------------------+
//| Script program start function                                    |
//+------------------------------------------------------------------+
void closeAllPairs ()
{
   TradeManager trades;
   trades.HedgeAll();
   trades.ClosePending();
   trades.CloseByHedge();
}
//+------------------------------------------------------------------+
/////////////////////////////////////////////////////////////////////
void printLines () {
   printLine("avgPriceBuy", avgPrice[OP_BUY], 4, clrAqua);
   printLine("avgPriceSell", avgPrice[OP_SELL], 4, clrOrange);
   printLabel("avgPriceBuyLabel", DoubleToString(lotsOpen[OP_BUY], 2) + " buy lots, " + DoubleToString(avgPrice[OP_BUY], Digits), getLabelTime(), avgPrice[OP_BUY], clrAqua, ANCHOR_LOWER);
   printLabel("avgPriceSellLabel", DoubleToString(lotsOpen[OP_SELL], 2) + " sell lots, " + DoubleToString(avgPrice[OP_SELL], Digits), getLabelTime(), avgPrice[OP_SELL], clrOrange, ANCHOR_UPPER);
   //printLine("avgPricePendingBuy", avgPricePending[OP_BUY], 2, clrAqua);
   //printLine("avgPricePendingSell", avgPricePending[OP_SELL], 2, clrOrange);
}
/////////////////////////////////////////////////////////////////////////
void printLine (string objName, double price, int width, color col) {
   if (price > 0) {
      if (ObjectFind(objName) < 0) {
         ObjectCreate(objName, OBJ_HLINE, 0, Time[0], price);
         ObjectSet(objName, OBJPROP_COLOR, col);
         ObjectSet(objName, OBJPROP_WIDTH, width);
      }
      if (ObjectGet(objName, OBJPROP_PRICE1) != price) ObjectSet(objName, OBJPROP_PRICE1, price);
   }
   else { if (ObjectFind(objName) == 0) ObjectDelete(objName); }
} 
///////////////////////////////////////////////////////////////////////////
datetime getLabelTime () {
   datetime time;
   int shift;
   double price;
   int chartId = 0;
   ChartXYToTimePrice(0, 1, 0, chartId, time, price);
   shift = (int) iBarShift(Symbol(), Period(), time, false) / 2;
   return(Time[shift]);
}
//////////////////////////////////////////////////////////////////////////
void printLabel (string objName, string text, datetime time, double price, color col, int anchor) {
   if (price == 0) {
      ObjectDelete(0, objName);
      return;
   }
   if (ObjectFind(0, objName) < 0) {
      ObjectCreate(objName, OBJ_TEXT, 0, time, price); 
      ObjectSetInteger(0, objName, OBJPROP_ANCHOR, anchor);
   }
   if (cd(ObjectGetDouble(0, objName, OBJPROP_PRICE1), NEQ, price)) ObjectSetDouble(0, objName, OBJPROP_PRICE1, price);
   if (ObjectGetInteger(0, objName, OBJPROP_TIME1) != time) ObjectSetInteger(0, objName, OBJPROP_TIME1, time);
   if (ObjectGetString(0, objName, OBJPROP_TEXT) != text) ObjectSetText(objName, text, 10, "Arial black", col);
}
////////////////////////////////////////////////////////////////////////////
bool cd (double a,int type,double b){
  // See http://forum.mql4.com/45053/page4
  // 0 compare doesn't need this function
  switch(type){
    case LT: return(b-a>Point/2.);
    case GT: return(a-b>Point/2.);
    case GTE: return(a-b>-Point/2.);
    case LTE: return(b-a>-Point/2.);
    case EQ: return(!(MathAbs(a-b)>Point/2.));
    case NEQ: return(MathAbs(a-b)>Point/2.);
  }
  return(-1);
}
///////////////////////////////////////////////////////////////////////
void printPl (color col, bool overridTiming = false) {
   if (WritePlEveryXMin == 0) return;
   if (TimeCurrent() < (datetime)globVarGet("lastPlPrintTime") + WritePlEveryXMin*60  && overridTiming == false) return;
   string str = "$";
   str = str + DoubleToString(plOpen, 0); // PL total
   if (overridTiming) str = str + "/$" + DoubleToString(globVarGet("maxDd"), 0);
   str = str + ", " + DoubleToString(lotsOpen[OP_BUY]+lotsOpen[OP_SELL], 2) + " (" + DoubleToString(lotsOpen[OP_BUY], 2) + "/" + DoubleToString(lotsOpen[OP_SELL], 2) + ")"; // Lots open total (b/s)
   str = str + ", " + IntegerToString(orderCount[OP_BUY]+orderCount[OP_SELL]) + " (" + IntegerToString(orderCount[OP_BUY]) + "/" + IntegerToString(orderCount[OP_SELL]) + ")";
   int printPointDir;
   if (globVarGet("lastPrintPoint") >= 0) printPointDir = -1; else printPointDir = 1;
   printPoint("PlInfo_"+IntegerToString((int)TimeCurrent()), str, printPointDir, TimeCurrent(), col);
   globVarSet("lastPrintPoint", printPointDir);
   globVarSet("lastPlPrintTime", TimeCurrent());
}
////////////////////////////////////////////////////////////////////////////////////////
void printPoint (string objName, string text, int location, datetime time, color col) {
   int shift = iBarShift(Symbol(), Period(), time, false);
   if (shift >= Bars-2) return;
   double objDistance = 2 * iATR(Symbol(), Period(), MathMin(50, Bars-shift-1), shift);
   double price;
   int anchorPoint;
   if (location == -1) { anchorPoint = ANCHOR_UPPER; price = Low[shift] - objDistance; }
   else { anchorPoint = ANCHOR_LOWER; price = High[shift] + objDistance; }
   ObjectCreate(objName, OBJ_TEXT, 0, time, price); 
   ObjectSetText(objName, text, 9, "Arial", col);
   ObjectSetInteger(0, objName, OBJPROP_ANCHOR, anchorPoint);
}
